@@ -1,4 +1,4 @@
-// This file Copyright © 2005-2023 Transmission authors and contributors.
+// This file Copyright © Transmission authors and contributors.
 // It may be used under the MIT (SPDX: MIT) license.
 // License text can be found in the licenses/ folder.
 
@@ -12,11 +12,14 @@
 #include "PrefsDialog.h"
 #include "Session.h"
 #include "Torrent.h"
-#include "TorrentCellRenderer.h"
 #include "Utils.h"
 
+#if !GTKMM_CHECK_VERSION(4, 0, 0)
+#include "TorrentCellRenderer.h"
+#endif
+
 #include <libtransmission/transmission.h>
-#include <libtransmission/utils.h> // tr_formatter_speed_KBps()
+#include <libtransmission/values.h>
 
 #include <gdkmm/cursor.h>
 #include <gdkmm/rectangle.h>
@@ -36,18 +39,20 @@
 #include <gtkmm/scrolledwindow.h>
 #include <gtkmm/togglebutton.h>
 #include <gtkmm/treemodel.h>
-#include <gtkmm/treeselection.h>
 #include <gtkmm/treeview.h>
-#include <gtkmm/treeviewcolumn.h>
 #include <gtkmm/widget.h>
 #include <gtkmm/window.h>
 
 #if GTKMM_CHECK_VERSION(4, 0, 0)
+#include <gtkmm/listitemfactory.h>
+#include <gtkmm/multiselection.h>
 #include <gtkmm/popovermenu.h>
 #else
 #include <gdkmm/display.h>
 #include <gdkmm/window.h>
 #include <gtkmm/menu.h>
+#include <gtkmm/treeselection.h>
+#include <gtkmm/treeviewcolumn.h>
 #endif
 
 #include <array>
@@ -56,6 +61,7 @@
 
 using namespace std::string_literals;
 using namespace std::string_view_literals;
+using namespace libtransmission::Values;
 
 using VariantInt = Glib::Variant<int>;
 using VariantDouble = Glib::Variant<double>;
@@ -78,17 +84,22 @@ class MainWindow::Impl
         Glib::RefPtr<Gio::Menu> section;
     };
 
+    using TorrentView = IF_GTKMM4(Gtk::ListView, Gtk::TreeView);
+    using TorrentViewSelection = IF_GTKMM4(Gtk::MultiSelection, Gtk::TreeSelection);
+
 public:
     Impl(
         MainWindow& window,
         Glib::RefPtr<Gtk::Builder> const& builder,
         Glib::RefPtr<Gio::ActionGroup> const& actions,
         Glib::RefPtr<Session> const& core);
+    Impl(Impl&&) = delete;
+    Impl(Impl const&) = delete;
+    Impl& operator=(Impl&&) = delete;
+    Impl& operator=(Impl const&) = delete;
     ~Impl();
 
-    TR_DISABLE_COPY_MOVE(Impl)
-
-    [[nodiscard]] Glib::RefPtr<Gtk::TreeSelection> get_selection() const;
+    [[nodiscard]] Glib::RefPtr<TorrentViewSelection> get_selection() const;
 
     void refresh();
 
@@ -100,7 +111,7 @@ public:
     }
 
 private:
-    void init_view(Gtk::TreeView* view, Glib::RefPtr<FilterBar::Model> const& model);
+    void init_view(TorrentView* view, Glib::RefPtr<FilterBar::Model> const& model);
 
     Glib::RefPtr<Gio::MenuModel> createOptionsMenu();
     Glib::RefPtr<Gio::MenuModel> createSpeedMenu(Glib::RefPtr<Gio::SimpleActionGroup> const& actions, tr_direction dir);
@@ -138,11 +149,17 @@ private:
     std::array<OptionMenuInfo, 2> speed_menu_info_;
     OptionMenuInfo ratio_menu_info_;
 
+#if GTKMM_CHECK_VERSION(4, 0, 0)
+    Glib::RefPtr<Gtk::ListItemFactory> item_factory_compact_;
+    Glib::RefPtr<Gtk::ListItemFactory> item_factory_full_;
+    Glib::RefPtr<Gtk::MultiSelection> selection_;
+#else
     TorrentCellRenderer* renderer_ = nullptr;
     Gtk::TreeViewColumn* column_ = nullptr;
+#endif
 
     Gtk::ScrolledWindow* scroll_ = nullptr;
-    Gtk::TreeView* view_ = nullptr;
+    TorrentView* view_ = nullptr;
     Gtk::Widget* toolbar_ = nullptr;
     FilterBar* filter_;
     Gtk::Widget* status_ = nullptr;
@@ -163,13 +180,20 @@ void MainWindow::Impl::on_popup_menu([[maybe_unused]] double event_x, [[maybe_un
 {
     if (popup_menu_ == nullptr)
     {
-        auto const menu = gtr_action_get_object<Gio::Menu>("main-window-popup");
+        auto const menu = gtr_action_get_object<Gio::Menu>(GTR_KEY_main_window_popup);
 
 #if GTKMM_CHECK_VERSION(4, 0, 0)
         popup_menu_ = Gtk::make_managed<Gtk::PopoverMenu>(menu, Gtk::PopoverMenu::Flags::NESTED);
-        popup_menu_->set_parent(window_);
+        popup_menu_->set_parent(*view_);
         popup_menu_->set_has_arrow(false);
-        popup_menu_->set_halign(window_.get_direction() == Gtk::TextDirection::RTL ? Gtk::Align::END : Gtk::Align::START);
+        popup_menu_->set_halign(view_->get_direction() == Gtk::TextDirection::RTL ? Gtk::Align::END : Gtk::Align::START);
+
+        view_->signal_destroy().connect(
+            [this]()
+            {
+                popup_menu_->unparent();
+                popup_menu_ = nullptr;
+            });
 #else
         popup_menu_ = Gtk::make_managed<Gtk::Menu>(menu);
         popup_menu_->attach_to_widget(window_);
@@ -177,13 +201,7 @@ void MainWindow::Impl::on_popup_menu([[maybe_unused]] double event_x, [[maybe_un
     }
 
 #if GTKMM_CHECK_VERSION(4, 0, 0)
-    int view_x = 0;
-    int view_y = 0;
-    view_->convert_bin_window_to_widget_coords(static_cast<int>(event_x), static_cast<int>(event_y), view_x, view_y);
-    double window_x = 0;
-    double window_y = 0;
-    view_->translate_coordinates(window_, view_x, view_y, window_x, window_y);
-    popup_menu_->set_pointing_to(Gdk::Rectangle(window_x, window_y, 1, 1));
+    popup_menu_->set_pointing_to({ static_cast<int>(event_x), static_cast<int>(event_y), 1, 1 });
     popup_menu_->popup();
 #else
     popup_menu_->popup_at_pointer(nullptr);
@@ -193,6 +211,38 @@ void MainWindow::Impl::on_popup_menu([[maybe_unused]] double event_x, [[maybe_un
 namespace
 {
 
+#if GTKMM_CHECK_VERSION(4, 0, 0)
+
+class GtrStrvBuilderDeleter
+{
+public:
+    void operator()(GStrvBuilder* builder) const
+    {
+        if (builder != nullptr)
+        {
+            g_strv_builder_unref(builder);
+        }
+    }
+};
+
+using GtrStrvBuilderPtr = std::unique_ptr<GStrvBuilder, GtrStrvBuilderDeleter>;
+
+GStrv gtr_strv_join(GObject* /*object*/, GStrv lhs, GStrv rhs)
+{
+    auto const builder = GtrStrvBuilderPtr(g_strv_builder_new());
+    if (builder == nullptr)
+    {
+        return nullptr;
+    }
+
+    g_strv_builder_addv(builder.get(), const_cast<char const**>(lhs)); // NOLINT(cppcoreguidelines-pro-type-const-cast)
+    g_strv_builder_addv(builder.get(), const_cast<char const**>(rhs)); // NOLINT(cppcoreguidelines-pro-type-const-cast)
+
+    return g_strv_builder_end(builder.get());
+}
+
+#else
+
 bool tree_view_search_equal_func(
     Glib::RefPtr<Gtk::TreeModel> const& /*model*/,
     int /*column*/,
@@ -205,10 +255,35 @@ bool tree_view_search_equal_func(
     return name.find(key.lowercase()) == Glib::ustring::npos;
 }
 
+#endif
+
 } // namespace
 
-void MainWindow::Impl::init_view(Gtk::TreeView* view, Glib::RefPtr<FilterBar::Model> const& model)
+void MainWindow::Impl::init_view(TorrentView* view, Glib::RefPtr<FilterBar::Model> const& model)
 {
+#if GTKMM_CHECK_VERSION(4, 0, 0)
+    auto const create_builder_list_item_factory = [](std::string const& filename)
+    {
+        auto builder_scope = Glib::wrap(G_OBJECT(gtk_builder_cscope_new()));
+        gtk_builder_cscope_add_callback(GTK_BUILDER_CSCOPE(builder_scope->gobj()), gtr_strv_join);
+
+        return Glib::wrap(gtk_builder_list_item_factory_new_from_resource(
+            GTK_BUILDER_SCOPE(builder_scope->gobj()),
+            gtr_get_full_resource_path(filename).c_str()));
+    };
+
+    item_factory_compact_ = create_builder_list_item_factory("TorrentListItemCompact.ui"s);
+    item_factory_full_ = create_builder_list_item_factory("TorrentListItemFull.ui"s);
+
+    view->signal_activate().connect([](guint /*position*/) { gtr_action_activate(GTR_KEY_show_torrent_properties); });
+
+    selection_ = Gtk::MultiSelection::create(model);
+    selection_->signal_selection_changed().connect([this](guint /*position*/, guint /*n_items*/)
+                                                   { signal_selection_changed_.emit(); });
+
+    view->set_factory(gtr_pref_flag_get(TR_KEY_compact_view) ? item_factory_compact_ : item_factory_full_);
+    view->set_model(selection_);
+#else
     static auto const& torrent_cols = Torrent::get_columns();
 
     view->set_search_column(torrent_cols.name_collated);
@@ -220,27 +295,27 @@ void MainWindow::Impl::init_view(Gtk::TreeView* view, Glib::RefPtr<FilterBar::Mo
     column_->pack_start(*renderer_, false);
     column_->add_attribute(renderer_->property_torrent(), torrent_cols.self);
 
-#if !GTKMM_CHECK_VERSION(4, 0, 0)
     view->signal_popup_menu().connect_notify([this]() { on_popup_menu(0, 0); });
+    view->signal_row_activated().connect([](auto const& /*path*/, auto* /*column*/)
+                                         { gtr_action_activate(GTR_KEY_show_torrent_properties); });
+
+    view->set_model(model);
+
+    view->get_selection()->signal_changed().connect([this]() { signal_selection_changed_.emit(); });
 #endif
-    setup_tree_view_button_event_handling(
+
+    setup_item_view_button_event_handling(
         *view,
         [this, view](guint /*button*/, TrGdkModifierType /*state*/, double view_x, double view_y, bool context_menu_requested)
         {
-            return on_tree_view_button_pressed(
+            return on_item_view_button_pressed(
                 *view,
                 view_x,
                 view_y,
                 context_menu_requested,
                 sigc::mem_fun(*this, &Impl::on_popup_menu));
         },
-        [view](double view_x, double view_y) { return on_tree_view_button_released(*view, view_x, view_y); });
-    view->signal_row_activated().connect([](auto const& /*path*/, auto* /*column*/)
-                                         { gtr_action_activate("show-torrent-properties"); });
-
-    view->set_model(IF_GTKMM4(ListModelAdapter::create<Torrent>(model), model));
-
-    view->get_selection()->signal_changed().connect([this]() { signal_selection_changed_.emit(); });
+        [view](double view_x, double view_y) { return on_item_view_button_released(*view, view_x, view_y); });
 }
 
 void MainWindow::Impl::prefsChanged(tr_quark const key)
@@ -248,14 +323,18 @@ void MainWindow::Impl::prefsChanged(tr_quark const key)
     switch (key)
     {
     case TR_KEY_compact_view:
+#if GTKMM_CHECK_VERSION(4, 0, 0)
+        view_->set_factory(gtr_pref_flag_get(key) ? item_factory_compact_ : item_factory_full_);
+#else
         renderer_->property_compact() = gtr_pref_flag_get(key);
         /* since the cell size has changed, we need gtktreeview to revalidate
          * its fixed-height mode values. Unfortunately there's not an API call
-         * for that, but this seems to work for both GTK 3 and 4 */
+         * for that, but this seems to work */
         view_->set_fixed_height_mode(false);
         view_->set_row_separator_func({});
         view_->unset_row_separator_func();
         view_->set_fixed_height_mode(true);
+#endif
         break;
 
     case TR_KEY_show_statusbar:
@@ -300,11 +379,13 @@ void MainWindow::Impl::syncAltSpeedButton()
 {
     bool const b = gtr_pref_flag_get(TR_KEY_alt_speed_enabled);
     alt_speed_button_->set_active(b);
-    alt_speed_button_->set_tooltip_text(fmt::format(
-        b ? _("Click to disable Alternative Speed Limits\n ({download_speed} down, {upload_speed} up)") :
-            _("Click to enable Alternative Speed Limits\n ({download_speed} down, {upload_speed} up)"),
-        fmt::arg("download_speed", tr_formatter_speed_KBps(gtr_pref_int_get(TR_KEY_alt_speed_down))),
-        fmt::arg("upload_speed", tr_formatter_speed_KBps(gtr_pref_int_get(TR_KEY_alt_speed_up)))));
+    alt_speed_button_->set_tooltip_text(
+        fmt::format(
+            fmt::runtime(
+                b ? _("Click to disable Alternative Speed Limits\n ({download_speed} down, {upload_speed} up)") :
+                    _("Click to enable Alternative Speed Limits\n ({download_speed} down, {upload_speed} up)")),
+            fmt::arg("download_speed", Speed{ gtr_pref_int_get(TR_KEY_alt_speed_down), Speed::Units::KByps }.to_string()),
+            fmt::arg("upload_speed", Speed{ gtr_pref_int_get(TR_KEY_alt_speed_up), Speed::Units::KByps }.to_string())));
 }
 
 void MainWindow::Impl::alt_speed_toggled_cb()
@@ -375,7 +456,7 @@ Glib::RefPtr<Gio::MenuModel> MainWindow::Impl::createSpeedMenu(
 
     for (auto const KBps : { 50, 100, 250, 500, 1000, 2500, 5000, 10000 })
     {
-        auto item = Gio::MenuItem::create(tr_formatter_speed_KBps(KBps), full_stock_action_name);
+        auto item = Gio::MenuItem::create(Speed{ KBps, Speed::Units::KByps }.to_string(), full_stock_action_name);
         item->set_action_and_target(full_stock_action_name, VariantInt::create(KBps));
         section->append_item(item);
     }
@@ -490,17 +571,19 @@ void MainWindow::Impl::onOptionsClicked()
 
     update_menu(
         speed_menu_info_[TR_DOWN],
-        tr_formatter_speed_KBps(gtr_pref_int_get(TR_KEY_speed_limit_down)),
+        Speed{ gtr_pref_int_get(TR_KEY_speed_limit_down), Speed::Units::KByps }.to_string(),
         TR_KEY_speed_limit_down_enabled);
 
     update_menu(
         speed_menu_info_[TR_UP],
-        tr_formatter_speed_KBps(gtr_pref_int_get(TR_KEY_speed_limit_up)),
+        Speed{ gtr_pref_int_get(TR_KEY_speed_limit_up), Speed::Units::KByps }.to_string(),
         TR_KEY_speed_limit_up_enabled);
 
     update_menu(
         ratio_menu_info_,
-        fmt::format(_("Stop at Ratio ({ratio})"), fmt::arg("ratio", tr_strlratio(gtr_pref_double_get(TR_KEY_ratio_limit)))),
+        fmt::format(
+            fmt::runtime(_("Stop at Ratio ({ratio})")),
+            fmt::arg("ratio", tr_strlratio(gtr_pref_double_get(TR_KEY_ratio_limit)))),
         TR_KEY_ratio_limit_enabled);
 }
 
@@ -577,7 +660,7 @@ MainWindow::Impl::Impl(
     : window_(window)
     , core_(core)
     , scroll_(gtr_get_widget<Gtk::ScrolledWindow>(builder, "torrents_view_scroll"))
-    , view_(gtr_get_widget<Gtk::TreeView>(builder, "torrents_view"))
+    , view_(gtr_get_widget<TorrentView>(builder, "torrents_view"))
     , toolbar_(gtr_get_widget<Gtk::Widget>(builder, "toolbar"))
     , filter_(gtr_get_widget_derived<FilterBar>(builder, "filterbar", core_))
     , status_(gtr_get_widget<Gtk::Widget>(builder, "statusbar"))
@@ -681,13 +764,13 @@ void MainWindow::Impl::updateStats()
     if (auto const pch = gtr_pref_string_get(TR_KEY_statusbar_stats); pch == "session-ratio")
     {
         auto const stats = tr_sessionGetStats(session);
-        buf = fmt::format(_("Ratio: {ratio}"), fmt::arg("ratio", tr_strlratio(stats.ratio)));
+        buf = fmt::format(fmt::runtime(_("Ratio: {ratio}")), fmt::arg("ratio", tr_strlratio(stats.ratio)));
     }
     else if (pch == "session-transfer")
     {
         auto const stats = tr_sessionGetStats(session);
         buf = fmt::format(
-            C_("current session totals", "Down: {downloaded_size}, Up: {uploaded_size}"),
+            fmt::runtime(C_("current session totals", "Down: {downloaded_size}, Up: {uploaded_size}")),
             fmt::arg("downloaded_size", tr_strlsize(stats.downloadedBytes)),
             fmt::arg("uploaded_size", tr_strlsize(stats.uploadedBytes)));
     }
@@ -695,14 +778,14 @@ void MainWindow::Impl::updateStats()
     {
         auto const stats = tr_sessionGetCumulativeStats(session);
         buf = fmt::format(
-            C_("all-time totals", "Down: {downloaded_size}, Up: {uploaded_size}"),
+            fmt::runtime(C_("all-time totals", "Down: {downloaded_size}, Up: {uploaded_size}")),
             fmt::arg("downloaded_size", tr_strlsize(stats.downloadedBytes)),
             fmt::arg("uploaded_size", tr_strlsize(stats.uploadedBytes)));
     }
     else /* default is total-ratio */
     {
         auto const stats = tr_sessionGetCumulativeStats(session);
-        buf = fmt::format(_("Ratio: {ratio}"), fmt::arg("ratio", tr_strlratio(stats.ratio)));
+        buf = fmt::format(fmt::runtime(_("Ratio: {ratio}")), fmt::arg("ratio", tr_strlratio(stats.ratio)));
     }
 
     stats_lb_->set_text(buf);
@@ -715,9 +798,9 @@ void MainWindow::Impl::updateSpeeds()
     if (session != nullptr)
     {
         auto dn_count = int{};
-        auto dn_speed = double{};
+        auto dn_speed = Speed{};
         auto up_count = int{};
-        auto up_speed = double{};
+        auto up_speed = Speed{};
 
         auto const model = core_->get_model();
         for (auto i = 0U, count = model->get_n_items(); i < count; ++i)
@@ -729,10 +812,10 @@ void MainWindow::Impl::updateSpeeds()
             up_speed += torrent->get_speed_up();
         }
 
-        dl_lb_->set_text(fmt::format(_("{download_speed} ▼"), fmt::arg("download_speed", tr_formatter_speed_KBps(dn_speed))));
+        dl_lb_->set_text(fmt::format(fmt::runtime(_("{download_speed} ▼")), fmt::arg("download_speed", dn_speed.to_string())));
         dl_lb_->set_visible(dn_count > 0);
 
-        ul_lb_->set_text(fmt::format(_("{upload_speed} ▲"), fmt::arg("upload_speed", tr_formatter_speed_KBps(up_speed))));
+        ul_lb_->set_text(fmt::format(fmt::runtime(_("{upload_speed} ▲")), fmt::arg("upload_speed", up_speed.to_string())));
         ul_lb_->set_visible(dn_count > 0 || up_count > 0);
     }
 }
@@ -751,9 +834,9 @@ void MainWindow::Impl::refresh()
     }
 }
 
-Glib::RefPtr<Gtk::TreeSelection> MainWindow::Impl::get_selection() const
+Glib::RefPtr<MainWindow::Impl::TorrentViewSelection> MainWindow::Impl::get_selection() const
 {
-    return view_->get_selection();
+    return IF_GTKMM4(selection_, view_->get_selection());
 }
 
 void MainWindow::for_each_selected_torrent(std::function<void(Glib::RefPtr<Torrent> const&)> const& callback) const
@@ -763,12 +846,23 @@ void MainWindow::for_each_selected_torrent(std::function<void(Glib::RefPtr<Torre
 
 bool MainWindow::for_each_selected_torrent_until(std::function<bool(Glib::RefPtr<Torrent> const&)> const& callback) const
 {
-    static auto const& self_col = Torrent::get_columns().self;
-
     auto const selection = impl_->get_selection();
     auto const model = selection->get_model();
     bool result = false;
 
+#if GTKMM_CHECK_VERSION(4, 0, 0)
+    auto const selected_items = selection->get_selection(); // TODO(C++20): Move into the `for`
+    for (auto const position : *selected_items)
+    {
+        if (callback(gtr_ptr_dynamic_cast<Torrent>(model->get_object(position))))
+        {
+            result = true;
+            break;
+        }
+    }
+#else
+    static auto const& self_col = Torrent::get_columns().self;
+
     for (auto const& path : selection->get_selected_rows())
     {
         auto const torrent = Glib::make_refptr_for_instance(model->get_iter(path)->get_value(self_col));
@@ -779,6 +873,7 @@ bool MainWindow::for_each_selected_torrent_until(std::function<bool(Glib::RefPtr
             break;
         }
     }
+#endif
 
     return result;
 }
@@ -798,7 +893,7 @@ void MainWindow::set_busy(bool isBusy)
     if (get_realized())
     {
 #if GTKMM_CHECK_VERSION(4, 0, 0)
-        auto const cursor = isBusy ? Gdk::Cursor::create("wait") : Glib::RefPtr<Gdk::Cursor>();
+        auto const cursor = isBusy ? Gdk::Cursor::create(Glib::ustring("wait")) : Glib::RefPtr<Gdk::Cursor>();
         set_cursor(cursor);
 #else
         auto const display = get_display();
