@@ -1,4 +1,4 @@
-// This file Copyright © 2010-2023 Mnemosyne LLC.
+// This file Copyright © Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
@@ -9,31 +9,38 @@
 #error only libtransmission should #include this header.
 #endif
 
-#include <atomic>
 #include <cstddef> // size_t
 #include <cstdint> // uint32_t
 #include <ctime>
 #include <functional>
+#include <memory>
+#include <optional>
 #include <string_view>
 #include <vector>
 
-#include "transmission.h"
+#ifdef _WIN32
+#include <ws2tcpip.h>
+#else
+#include <sys/socket.h> // socklen_t
+#endif
 
-#include "interned-string.h"
-#include "net.h"
+#include "libtransmission/transmission.h"
 
-class tr_announcer;
+#include "libtransmission/interned-string.h"
+#include "libtransmission/peer-mgr.h"
+
+struct tr_address;
 class tr_announcer_udp;
+struct tr_session;
+struct tr_torrent;
 struct tr_torrent_announcer;
 
 // --- Tracker Publish / Subscribe
 
-struct tr_pex;
-
 /** @brief Notification object to tell listeners about announce or scrape occurrences */
 struct tr_tracker_event
 {
-    enum class Type
+    enum class Type : uint8_t
     {
         Error,
         ErrorClear,
@@ -53,8 +60,9 @@ struct tr_tracker_event
     std::vector<tr_pex> pex;
 
     // for Peers and Counts events
-    int leechers;
-    int seeders;
+    std::optional<int64_t> leechers;
+    std::optional<int64_t> seeders;
+    std::optional<int64_t> downloaders;
 };
 
 using tr_tracker_callback = std::function<void(tr_torrent&, tr_tracker_event const*)>;
@@ -62,10 +70,7 @@ using tr_tracker_callback = std::function<void(tr_torrent&, tr_tracker_event con
 class tr_announcer
 {
 public:
-    [[nodiscard]] static std::unique_ptr<tr_announcer> create(
-        tr_session* session,
-        tr_announcer_udp&,
-        std::atomic<size_t>& n_pending_stops);
+    [[nodiscard]] static std::unique_ptr<tr_announcer> create(tr_session* session, tr_announcer_udp& announcer_udp);
     virtual ~tr_announcer() = default;
 
     virtual tr_torrent_announcer* addTorrent(tr_torrent*, tr_tracker_callback callback) = 0;
@@ -74,30 +79,30 @@ public:
     virtual void resetTorrent(tr_torrent* tor) = 0;
     virtual void removeTorrent(tr_torrent* tor) = 0;
     virtual void startShutdown() = 0;
-};
 
-std::unique_ptr<tr_announcer> tr_announcerCreate(tr_session* session);
+    virtual void upkeep() = 0;
+};
 
 // --- For torrent customers
 
-void tr_announcerChangeMyPort(tr_torrent*);
+void tr_announcerChangeMyPort(tr_torrent* tor);
 
-bool tr_announcerCanManualAnnounce(tr_torrent const*);
+bool tr_announcerCanManualAnnounce(tr_torrent const* tor);
 
-void tr_announcerManualAnnounce(tr_torrent*);
+void tr_announcerManualAnnounce(tr_torrent* tor);
 
-void tr_announcerTorrentCompleted(tr_torrent*);
+void tr_announcerTorrentCompleted(tr_torrent* tor);
 
-enum
+enum : uint8_t
 {
     TR_ANN_UP,
     TR_ANN_DOWN,
     TR_ANN_CORRUPT
 };
 
-void tr_announcerAddBytes(tr_torrent*, int type, uint32_t n_bytes);
+void tr_announcerAddBytes(tr_torrent* tor, int type, uint32_t n_bytes);
 
-time_t tr_announcerNextManualAnnounce(tr_torrent const*);
+time_t tr_announcerNextManualAnnounce(tr_torrent const* tor);
 
 tr_tracker_view tr_announcerTracker(tr_torrent const* torrent, size_t nth);
 
@@ -105,7 +110,7 @@ size_t tr_announcerTrackerCount(tr_torrent const* tor);
 
 // --- ANNOUNCE
 
-enum tr_announce_event
+enum tr_announce_event : uint8_t
 {
     /* Note: the ordering of this enum's values is important to
      * announcer.c's tr_tier.announce_event_priority. If changing
@@ -135,12 +140,12 @@ public:
     public:
         virtual ~Mediator() noexcept = default;
         virtual void sendto(void const* buf, size_t buflen, sockaddr const* addr, socklen_t addrlen) = 0;
-        [[nodiscard]] virtual std::optional<tr_address> announceIP() const = 0;
+        [[nodiscard]] virtual std::optional<tr_address> announce_ip() const = 0;
     };
 
     virtual ~tr_announcer_udp() noexcept = default;
 
-    [[nodiscard]] static std::unique_ptr<tr_announcer_udp> create(Mediator&);
+    [[nodiscard]] static std::unique_ptr<tr_announcer_udp> create(Mediator& mediator);
 
     virtual void announce(tr_announce_request const& request, tr_announce_response_func on_response) = 0;
 
@@ -150,5 +155,7 @@ public:
 
     // @brief process an incoming udp message if it's a tracker response.
     // @return true if msg was a tracker response; false otherwise
-    virtual bool handleMessage(uint8_t const* msg, size_t msglen) = 0;
+    virtual bool handle_message(uint8_t const* msg, size_t msglen, struct sockaddr const* from, socklen_t fromlen) = 0;
+
+    [[nodiscard]] virtual bool is_idle() const noexcept = 0;
 };
