@@ -1,4 +1,4 @@
-// This file Copyright © 2022-2023 Mnemosyne LLC.
+// This file Copyright © Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
@@ -14,7 +14,7 @@
 #include <string_view>
 #include <utility> // make_pair
 
-#include "error.h"
+#include "libtransmission/error.h"
 
 namespace transmission::benc
 {
@@ -33,25 +33,29 @@ struct Handler
     class Context
     {
     public:
-        Context(char const* stream_begin_in, tr_error** error_in)
+        constexpr Context(char const* stream_begin_in, tr_error& error_in)
             : error{ error_in }
             , stream_begin_{ stream_begin_in }
         {
         }
-        [[nodiscard]] std::pair<long, long> tokenSpan() const
+
+        [[nodiscard]] constexpr std::pair<long, long> tokenSpan() const
         {
-            return std::make_pair(token_begin_ - stream_begin_, token_end_ - stream_begin_);
+            return { token_begin_ - stream_begin_, token_end_ - stream_begin_ };
         }
-        [[nodiscard]] auto raw() const
+
+        [[nodiscard]] constexpr auto raw() const
         {
             return std::string_view{ token_begin_, size_t(token_end_ - token_begin_) };
         }
-        void setTokenSpan(char const* a, size_t len)
+
+        constexpr void setTokenSpan(char const* a, size_t len)
         {
             token_begin_ = a;
             token_end_ = token_begin_ + len;
         }
-        tr_error** error = nullptr;
+
+        tr_error& error;
 
     private:
         char const* token_begin_ = nullptr;
@@ -115,21 +119,23 @@ struct BasicHandler : public Handler
         return true;
     }
 
-    constexpr auto key(size_t i) const
+    [[nodiscard]] constexpr auto key(size_t i) const
     {
         return keys_[i];
     }
 
-    constexpr auto depth() const
+    [[nodiscard]] constexpr auto depth() const
     {
         return depth_;
     }
 
-    constexpr auto currentKey() const
+    [[nodiscard]] constexpr auto currentKey() const
     {
         return key(depth());
     }
 
+    static auto constexpr ArrayKey = std::nullopt;
+
 protected:
     [[nodiscard]] std::string path() const
     {
@@ -137,12 +143,26 @@ protected:
         for (size_t i = 0; i <= depth(); ++i)
         {
             ret += '[';
-            ret += key(i);
+            ret += key(i).value_or(std::string_view{});
             ret += ']';
         }
         return ret;
     }
 
+    template<typename... Args>
+    [[nodiscard]] bool pathStartsWith(Args... args) const noexcept
+    {
+        auto i = 1U;
+        return (depth() >= sizeof...(args)) && ((key(i++) == args) && ...);
+    }
+
+    template<typename... Args>
+    [[nodiscard]] bool pathIs(Args... args) const noexcept
+    {
+        auto i = 1U;
+        return (depth() == sizeof...(args)) && ((key(i++) == args) && ...);
+    }
+
 private:
     constexpr void push() noexcept
     {
@@ -156,13 +176,13 @@ private:
     }
 
     size_t depth_ = 0;
-    std::array<std::string_view, MaxDepth> keys_;
+    std::array<std::optional<std::string_view>, MaxDepth> keys_;
 };
 
 template<std::size_t MaxDepth>
 struct ParserStack
 {
-    enum class ParentType
+    enum class ParentType : uint8_t
     {
         Array,
         Dict
@@ -205,27 +225,17 @@ struct ParserStack
         return depth > 0 && stack[depth].parent_type == ParentType::Dict && (stack[depth].n_children_walked % 2) == 0;
     }
 
-    constexpr std::optional<ParentType> parentType() const
+    std::optional<ParentType> pop(tr_error& error)
     {
         if (depth == 0)
         {
-            return {};
-        }
-
-        return stack[depth].parent_type;
-    }
-
-    std::optional<ParentType> pop(tr_error** error)
-    {
-        if (depth == 0)
-        {
-            tr_error_set(error, EILSEQ, "Cannot pop empty stack");
+            error.set(EILSEQ, "Cannot pop empty stack");
             return {};
         }
 
         if (stack[depth].parent_type == ParentType::Dict && ((stack[depth].n_children_walked % 2) != 0))
         {
-            tr_error_set(error, EILSEQ, "Premature end-of-dict found. Malformed benc?");
+            error.set(EILSEQ, "Premature end-of-dict found. Malformed benc?");
             return {};
         }
 
@@ -234,11 +244,11 @@ struct ParserStack
         return ret;
     }
 
-    bool push(ParentType parent_type, tr_error** error)
+    bool push(ParentType parent_type, tr_error& error)
     {
         if (depth + 1 >= std::size(stack))
         {
-            tr_error_set(error, E2BIG, "Max stack depth reached; unable to continue parsing");
+            error.set(E2BIG, "Max stack depth reached; unable to continue parsing");
             return false;
         }
 
@@ -254,11 +264,18 @@ bool parse(
     ParserStack<MaxDepth>& stack,
     Handler& handler,
     char const** setme_end = nullptr,
-    tr_error** error = nullptr)
+    tr_error* error = nullptr)
 {
+    // ensure `error` isn't a nullptr
+    auto local_error = tr_error{};
+    if (error == nullptr)
+    {
+        error = &local_error;
+    }
+
     stack.clear();
     auto const* const stream_begin = std::data(benc);
-    auto context = Handler::Context(stream_begin, error);
+    auto context = Handler::Context{ stream_begin, *error };
 
     int err = 0;
     for (;;)
@@ -279,8 +296,8 @@ bool parse(
         case 'i': // int
             if (auto const value = impl::ParseInt(&benc); !value)
             {
-                tr_error_set(error, err, "Malformed benc? Unable to parse integer");
                 err = EILSEQ;
+                error->set(err, "Malformed benc? Unable to parse integer");
             }
             else
             {
@@ -300,8 +317,8 @@ bool parse(
         case 'l': // list
         case 'd': // dict
             {
-                bool ok = benc.front() == 'l' ? stack.push(ParserStack<MaxDepth>::ParentType::Array, error) :
-                                                stack.push(ParserStack<MaxDepth>::ParentType::Dict, error);
+                bool ok = benc.front() == 'l' ? stack.push(ParserStack<MaxDepth>::ParentType::Array, *error) :
+                                                stack.push(ParserStack<MaxDepth>::ParentType::Dict, *error);
                 if (!ok)
                 {
                     err = EILSEQ;
@@ -322,7 +339,7 @@ bool parse(
         case 'e': // end of list or dict
             benc.remove_prefix(1);
 
-            if (auto const parent_type = stack.pop(error); !parent_type)
+            if (auto const parent_type = stack.pop(*error); !parent_type)
             {
                 err = EILSEQ;
             }
@@ -353,7 +370,7 @@ bool parse(
             if (auto const sv = impl::ParseString(&benc); !sv)
             {
                 err = EILSEQ;
-                tr_error_set(error, err, "Malformed benc? Unable to parse string");
+                error->set(err, "Malformed benc? Unable to parse string");
             }
             else
             {
@@ -380,6 +397,11 @@ bool parse(
         }
     }
 
+    if (setme_end != nullptr)
+    {
+        *setme_end = std::data(benc);
+    }
+
     if (err != 0)
     {
         errno = err;
@@ -389,7 +411,7 @@ bool parse(
     if (stack.depth != 0)
     {
         err = EILSEQ;
-        tr_error_set(error, err, "premature end-of-data reached");
+        error->set(err, "premature end-of-data reached");
         errno = err;
         return false;
     }
@@ -397,16 +419,11 @@ bool parse(
     if (stack.stack[0].n_children_walked == 0)
     {
         err = EILSEQ;
-        tr_error_set(error, err, "no bencoded data to parse");
+        error->set(err, "no bencoded data to parse");
         errno = err;
         return false;
     }
 
-    if (setme_end != nullptr)
-    {
-        *setme_end = std::data(benc);
-    }
-
     return true;
 }
 
