@@ -1,32 +1,37 @@
-// This file Copyright © 2010-2023 Mnemosyne LLC.
+// This file Copyright © Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
 
 #include <algorithm>
 #include <cerrno> // for ENOENT
-#include <optional>
+#include <cmath>
+#include <ctime> // time()
+#include <iterator>
 #include <set>
 #include <string>
 #include <string_view>
-#include <thread>
 #include <utility>
 #include <vector>
 
 #include <fmt/format.h>
 
-#include "transmission.h"
+#include "libtransmission/transmission.h"
 
-#include "crypto-utils.h"
-#include "error.h"
-#include "file.h"
-#include "log.h"
-#include "makemeta.h"
-#include "session.h" // TR_NAME
-#include "tr-assert.h"
-#include "utils.h" // for _()
-#include "variant.h"
-#include "version.h"
+#include "libtransmission/block-info.h" // tr_block_info
+#include "libtransmission/crypto-utils.h"
+#include "libtransmission/error.h"
+#include "libtransmission/file.h"
+#include "libtransmission/log.h"
+#include "libtransmission/makemeta.h"
+#include "libtransmission/quark.h" // TR_KEY_length, TR_KEY_a...
+#include "libtransmission/session.h" // TR_NAME
+#include "libtransmission/torrent-files.h"
+#include "libtransmission/tr-assert.h"
+#include "libtransmission/tr-strbuf.h" // tr_pathbuf
+#include "libtransmission/utils.h" // for _()
+#include "libtransmission/variant.h"
+#include "libtransmission/version.h"
 
 using namespace std::literals;
 
@@ -67,16 +72,16 @@ void walkTree(std::string_view const top, std::string_view const subpath, std::s
 
     auto path = tr_pathbuf{ top, '/', subpath };
     tr_sys_path_native_separators(std::data(path));
-    tr_error* error = nullptr;
+    auto error = tr_error{};
     auto const info = tr_sys_path_get_info(path, 0, &error);
-    if (error != nullptr)
+    if (error)
     {
-        tr_logAddWarn(fmt::format(
-            _("Skipping '{path}': {error} ({error_code})"),
-            fmt::arg("path", path),
-            fmt::arg("error", error->message),
-            fmt::arg("error_code", error->code)));
-        tr_error_free(error);
+        tr_logAddWarn(
+            fmt::format(
+                fmt::runtime(_("Skipping '{path}': {error} ({error_code})")),
+                fmt::arg("path", path),
+                fmt::arg("error", error.message()),
+                fmt::arg("error_code", error.code())));
     }
     if (!info)
     {
@@ -86,33 +91,16 @@ void walkTree(std::string_view const top, std::string_view const subpath, std::s
     switch (info->type)
     {
     case TR_SYS_PATH_IS_DIRECTORY:
-        if (tr_sys_dir_t odir = tr_sys_dir_open(path.c_str()); odir != TR_BAD_SYS_DIR)
+        for (auto const& name : tr_sys_dir_get_files(path))
         {
-            for (;;)
+            if (!std::empty(subpath))
             {
-                char const* const name = tr_sys_dir_read_name(odir);
-
-                if (name == nullptr)
-                {
-                    break;
-                }
-
-                if (name[0] == '.') // skip dotfiles
-                {
-                    continue;
-                }
-
-                if (!std::empty(subpath))
-                {
-                    walkTree(top, tr_pathbuf{ subpath, '/', name }, files);
-                }
-                else
-                {
-                    walkTree(top, name, files);
-                }
+                walkTree(top, tr_pathbuf{ subpath, '/', name }, files);
+            }
+            else
+            {
+                walkTree(top, name, files);
             }
-
-            tr_sys_dir_close(odir);
         }
         break;
 
@@ -147,49 +135,45 @@ tr_metainfo_builder::tr_metainfo_builder(std::string_view single_file_or_parent_
     : top_{ single_file_or_parent_directory }
 {
     files_ = findFiles(tr_sys_path_dirname(top_), tr_sys_path_basename(top_));
-    block_info_ = tr_block_info{ files_.totalSize(), defaultPieceSize(files_.totalSize()) };
+    block_info_ = tr_block_info{ files_.total_size(), default_piece_size(files_.total_size()) };
 }
 
-bool tr_metainfo_builder::isLegalPieceSize(uint32_t x)
+bool tr_metainfo_builder::set_piece_size(uint32_t piece_size) noexcept
 {
-    // It must be a power of two and at least 16KiB
-    static auto constexpr MinSize = uint32_t{ 1024U * 16U };
-    auto const is_power_of_two = (x & (x - 1)) == 0;
-    return x >= MinSize && is_power_of_two;
-}
-
-bool tr_metainfo_builder::setPieceSize(uint32_t piece_size) noexcept
-{
-    if (!isLegalPieceSize(piece_size))
+    if (!is_legal_piece_size(piece_size))
     {
         return false;
     }
 
-    block_info_ = tr_block_info{ files_.totalSize(), piece_size };
+    block_info_ = tr_block_info{ files_.total_size(), piece_size };
     return true;
 }
 
-bool tr_metainfo_builder::blockingMakeChecksums(tr_error** error)
+bool tr_metainfo_builder::blocking_make_checksums(tr_error* error)
 {
     checksum_piece_ = 0;
     cancel_ = false;
 
-    if (totalSize() == 0U)
+    if (total_size() == 0U)
     {
-        tr_error_set_from_errno(error, ENOENT);
+        if (error != nullptr)
+        {
+            error->set(ENOENT, "zero-length torrents are not allowed"sv);
+        }
+
         return false;
     }
 
-    auto hashes = std::vector<std::byte>(std::size(tr_sha1_digest_t{}) * pieceCount());
+    auto hashes = std::vector<std::byte>(std::size(tr_sha1_digest_t{}) * piece_count());
     auto* walk = std::data(hashes);
-    auto sha = tr_sha1::create();
+    auto sha = tr_sha1{};
 
     auto file_index = tr_file_index_t{ 0U };
     auto piece_index = tr_piece_index_t{ 0U };
-    auto total_remain = totalSize();
+    auto total_remain = total_size();
     auto off = uint64_t{ 0U };
 
-    auto buf = std::vector<char>(pieceSize());
+    auto buf = std::vector<char>(piece_size());
 
     auto const parent = tr_sys_path_dirname(top_);
     auto fd = tr_sys_file_open(
@@ -206,16 +190,16 @@ bool tr_metainfo_builder::blockingMakeChecksums(tr_error** error)
     {
         checksum_piece_ = piece_index;
 
-        TR_ASSERT(piece_index < pieceCount());
+        TR_ASSERT(piece_index < piece_count());
 
-        uint32_t const piece_size = block_info_.pieceSize(piece_index);
+        auto const piece_size = block_info_.piece_size(piece_index);
         buf.resize(piece_size);
         auto* bufptr = std::data(buf);
 
         auto left_in_piece = piece_size;
         while (left_in_piece > 0U)
         {
-            auto const n_this_pass = std::min(fileSize(file_index) - off, uint64_t{ left_in_piece });
+            auto const n_this_pass = std::min(file_size(file_index) - off, uint64_t{ left_in_piece });
             auto n_read = uint64_t{};
 
             (void)tr_sys_file_read(fd, bufptr, n_this_pass, &n_read, error);
@@ -223,13 +207,13 @@ bool tr_metainfo_builder::blockingMakeChecksums(tr_error** error)
             off += n_read;
             left_in_piece -= n_read;
 
-            if (off == fileSize(file_index))
+            if (off == file_size(file_index))
             {
                 off = 0;
                 tr_sys_file_close(fd);
                 fd = TR_BAD_SYS_FILE;
 
-                if (++file_index < fileCount())
+                if (++file_index < file_count())
                 {
                     fd = tr_sys_file_open(
                         tr_pathbuf{ parent, '/', path(file_index) },
@@ -246,10 +230,10 @@ bool tr_metainfo_builder::blockingMakeChecksums(tr_error** error)
 
         TR_ASSERT(bufptr - std::data(buf) == (int)piece_size);
         TR_ASSERT(left_in_piece == 0);
-        sha->add(std::data(buf), std::size(buf));
-        auto const digest = sha->finish();
+        sha.add(std::data(buf), std::size(buf));
+        auto const digest = sha.finish();
         walk = std::copy(std::begin(digest), std::end(digest), walk);
-        sha->clear();
+        sha.clear();
 
         total_remain -= piece_size;
         ++piece_index;
@@ -265,7 +249,11 @@ bool tr_metainfo_builder::blockingMakeChecksums(tr_error** error)
 
     if (cancel_)
     {
-        tr_error_set_from_errno(error, ECANCELED);
+        if (error != nullptr)
+        {
+            error->set_from_errno(ECANCELED);
+        }
+
         return false;
     }
 
@@ -273,7 +261,7 @@ bool tr_metainfo_builder::blockingMakeChecksums(tr_error** error)
     return true;
 }
 
-std::string tr_metainfo_builder::benc(tr_error** error) const
+std::string tr_metainfo_builder::benc(tr_error* error) const
 {
     TR_ASSERT_MSG(!std::empty(piece_hashes_), "did you forget to call makeChecksums() first?");
 
@@ -282,83 +270,65 @@ std::string tr_metainfo_builder::benc(tr_error** error) const
     auto const& source = this->source();
     auto const& webseeds = this->webseeds();
 
-    if (totalSize() == 0)
+    if (total_size() == 0)
     {
-        tr_error_set_from_errno(error, ENOENT);
+        if (error != nullptr)
+        {
+            error->set_from_errno(ENOENT);
+        }
+
         return {};
     }
 
-    auto top = tr_variant{};
-    tr_variantInitDict(&top, 8);
+    auto top = tr_variant::Map{ 8U };
 
-    // add the announce-list trackers
-    if (!std::empty(announceList()))
-    {
-        auto* const announce_list = tr_variantDictAddList(&top, TR_KEY_announce_list, 0);
-        tr_variant* tier_list = nullptr;
-        auto prev_tier = std::optional<tr_tracker_tier_t>{};
-        for (auto const& tracker : announceList())
-        {
-            if (!prev_tier || *prev_tier != tracker.tier)
-            {
-                tier_list = nullptr;
-            }
-
-            if (tier_list == nullptr)
-            {
-                prev_tier = tracker.tier;
-                tier_list = tr_variantListAddList(announce_list, 0);
-            }
-
-            tr_variantListAddStr(tier_list, tracker.announce);
-        }
-    }
+    // add the announce URLs
+    announce_list().add_to_map(top);
 
     // add the webseeds
-    if (!std::empty(webseeds))
+    if (auto const n_webseeds = std::size(webseeds); n_webseeds > 0U)
     {
-        auto* const url_list = tr_variantDictAddList(&top, TR_KEY_url_list, std::size(webseeds));
-
-        for (auto const& webseed : webseeds)
-        {
-            tr_variantListAddStr(url_list, webseed);
-        }
+        auto webseeds_vec = tr_variant::Vector{};
+        webseeds_vec.reserve(n_webseeds);
+        std::copy_n(std::cbegin(webseeds), n_webseeds, std::back_inserter(webseeds_vec));
+        top.try_emplace(TR_KEY_url_list, std::move(webseeds_vec));
     }
 
     // add the comment
     if (!std::empty(comment))
     {
-        tr_variantDictAddStr(&top, TR_KEY_comment, comment);
+        top.try_emplace(TR_KEY_comment, comment);
     }
 
     // maybe add some optional metainfo
     if (!anonymize)
     {
-        tr_variantDictAddStrView(&top, TR_KEY_created_by, TR_NAME "/" LONG_VERSION_STRING);
-        tr_variantDictAddInt(&top, TR_KEY_creation_date, time(nullptr));
+        top.try_emplace(TR_KEY_created_by, TR_NAME "/" LONG_VERSION_STRING);
+        top.try_emplace(TR_KEY_creation_date, time(nullptr));
     }
 
-    tr_variantDictAddStrView(&top, TR_KEY_encoding, "UTF-8");
+    top.try_emplace(TR_KEY_encoding, "UTF-8"sv);
 
-    auto* const info_dict = tr_variantDictAddDict(&top, TR_KEY_info, 5);
+    auto info_dict = tr_variant::Map{ 8U };
     auto const base = tr_sys_path_basename(top_);
 
     // "There is also a key `length` or a key `files`, but not both or neither.
     // If length is present then the download represents a single file,
     // otherwise it represents a set of files which go in a directory structure."
-    if (fileCount() == 1U && !tr_strvContains(path(0), '/'))
+    if (file_count() == 1U && !tr_strv_contains(path(0), '/'))
     {
-        tr_variantDictAddInt(info_dict, TR_KEY_length, fileSize(0));
+        info_dict.try_emplace(TR_KEY_length, file_size(0));
     }
     else
     {
-        auto const n_files = fileCount();
-        auto* const file_list = tr_variantDictAddList(info_dict, TR_KEY_files, n_files);
+        auto const n_files = file_count();
+        auto file_vec = tr_variant::Vector{};
+        file_vec.reserve(n_files);
 
-        for (tr_file_index_t i = 0; i < n_files; ++i)
+        for (tr_file_index_t i = 0U; i < n_files; ++i)
         {
-            auto* const file_dict = tr_variantListAddDict(file_list, 2);
-            tr_variantDictAddInt(file_dict, TR_KEY_length, fileSize(i));
+            auto file_map = tr_variant::Map{ 2U };
+            file_map.try_emplace(TR_KEY_length, file_size(i));
 
             auto subpath = std::string_view{ path(i) };
             if (!std::empty(base))
@@ -366,34 +336,56 @@ std::string tr_metainfo_builder::benc(tr_error** error) const
                 subpath.remove_prefix(std::size(base) + std::size("/"sv));
             }
 
-            auto* const path_list = tr_variantDictAddList(file_dict, TR_KEY_path, 0);
+            auto path_vec = tr_variant::Vector{};
             auto token = std::string_view{};
-            while (tr_strvSep(&subpath, &token, '/'))
+            while (tr_strv_sep(&subpath, &token, '/'))
             {
-                tr_variantListAddStr(path_list, token);
+                path_vec.emplace_back(token);
             }
+            file_map.try_emplace(TR_KEY_path, std::move(path_vec));
+
+            file_vec.emplace_back(std::move(file_map));
         }
+
+        info_dict.try_emplace(TR_KEY_files, std::move(file_vec));
     }
 
     if (!std::empty(base))
     {
-        tr_variantDictAddStr(info_dict, TR_KEY_name, base);
+        info_dict.try_emplace(TR_KEY_name, base);
     }
 
-    tr_variantDictAddInt(info_dict, TR_KEY_piece_length, pieceSize());
-    tr_variantDictAddRaw(info_dict, TR_KEY_pieces, std::data(piece_hashes_), std::size(piece_hashes_));
+    info_dict.try_emplace(TR_KEY_piece_length, piece_size());
+    info_dict.try_emplace(TR_KEY_pieces, tr_variant::make_raw(std::data(piece_hashes_), std::size(piece_hashes_)));
 
     if (is_private_)
     {
-        tr_variantDictAddInt(info_dict, TR_KEY_private, 1);
+        info_dict.try_emplace(TR_KEY_private, 1);
     }
 
     if (!std::empty(source))
     {
-        tr_variantDictAddStr(info_dict, TR_KEY_source, source_);
+        info_dict.try_emplace(TR_KEY_source, source_);
     }
 
-    auto ret = tr_variantToStr(&top, TR_VARIANT_FMT_BENC);
-    tr_variantClear(&top);
-    return ret;
+    top.try_emplace(TR_KEY_info, std::move(info_dict));
+    return tr_variant_serde::benc().to_string(tr_variant{ std::move(top) });
+}
+
+bool tr_metainfo_builder::save(std::string_view filename, tr_error* error) const
+{
+    return tr_file_save(filename, benc(error), error);
+}
+
+uint32_t tr_metainfo_builder::default_piece_size(uint64_t total_size) noexcept
+{
+    // Ideally, we want approximately 2^10 = 1024 pieces, give or take a few hundred pieces.
+    // So we subtract 10 from the log2 of total size.
+    // The ideal number of pieces is up for debate.
+    auto exp = (total_size > 0U ? std::log2(total_size) : 0) - 10;
+
+    // We want a piece size between 16KiB (2^14 bytes) and 16MiB (2^24 bytes) for maximum compatibility
+    exp = std::clamp(exp, 14., 24.);
+
+    return uint32_t{ 1U } << std::lround(exp);
 }
