@@ -1,4 +1,4 @@
-// This file Copyright © 2007-2023 Mnemosyne LLC.
+// This file Copyright © Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
@@ -11,7 +11,6 @@
 
 #include <functional>
 #include <memory>
-#include <tuple>
 #include <utility>
 
 struct event_base;
@@ -24,19 +23,32 @@ public:
     static std::unique_ptr<tr_session_thread> create();
     virtual ~tr_session_thread() = default;
 
-    [[nodiscard]] virtual struct event_base* eventBase() noexcept = 0;
+    [[nodiscard]] virtual struct event_base* event_base() noexcept = 0;
 
-    [[nodiscard]] virtual bool amInSessionThread() const noexcept = 0;
+    [[nodiscard]] virtual bool am_in_session_thread() const noexcept = 0;
+
+    virtual void queue(std::function<void(void)>&& func) = 0;
 
     virtual void run(std::function<void(void)>&& func) = 0;
 
+    template<typename Func, typename... Args>
+    void queue(Func&& func, Args&&... args)
+    {
+        // TODO(tearfur): Use C++20 P0780R2, GCC 9, clang 9
+        queue(
+            std::function<void(void)>{
+                [func = std::forward<Func>(func), args = std::make_tuple(std::forward<Args>(args)...)]()
+                { std::apply(std::move(func), std::move(args)); },
+            });
+    }
+
     template<typename Func, typename... Args>
     void run(Func&& func, Args&&... args)
     {
+        // TODO(tearfur): Use C++20 P0780R2, GCC 9, clang 9
         run(std::function<void(void)>{
-            [func = std::forward<Func&&>(func), args = std::make_tuple(std::forward<Args>(args)...)]()
-            {
-                std::apply(std::move(func), std::move(args));
-            } });
+            [func = std::forward<Func>(func), args = std::make_tuple(std::forward<Args>(args)...)]()
+            { std::apply(std::move(func), std::move(args)); },
+        });
     }
 };
