@@ -1,13 +1,13 @@
-// This file Copyright © 2009-2023 Juliusz Chroboczek.
+// This file Copyright © Juliusz Chroboczek.
 // It may be used under the MIT (SPDX: MIT) license.
 // License text can be found in the licenses/ folder.
 
 #include <algorithm>
-#include <cerrno>
+#include <array>
 #include <chrono>
-#include <cstdio>
-#include <cstdlib> // for abort()
-#include <cstring> // for memcpy()
+#include <cstddef>
+#include <cstdint> // uint16_t
+#include <cstring> // memcpy()
 #include <ctime>
 #include <deque>
 #include <fstream>
@@ -16,15 +16,15 @@
 #include <sstream>
 #include <string>
 #include <string_view>
-#include <tuple> // for std::tie()
+#include <tuple> // std::tie()
+#include <utility>
+#include <vector>
 
 #ifdef _WIN32
 #include <ws2tcpip.h>
 #undef gai_strerror
 #define gai_strerror gai_strerrorA
 #else
-#include <sys/time.h> // for `struct timezone`
-#include <sys/types.h>
 #include <sys/socket.h> /* socket(), bind() */
 #include <netdb.h>
 #include <netinet/in.h> /* sockaddr_in */
@@ -32,26 +32,26 @@
 
 #include <fmt/format.h>
 
-#include "transmission.h"
+#include "libtransmission/transmission.h"
 
-#include "crypto-utils.h"
-#include "file.h"
-#include "log.h"
-#include "net.h"
-#include "peer-mgr.h" // for tr_peerMgrCompactToPex()
-#include "timer.h"
-#include "tr-assert.h"
-#include "tr-dht.h"
-#include "tr-strbuf.h"
-#include "variant.h"
-#include "utils.h" // for tr_time(), _()
+#include "libtransmission/crypto-utils.h"
+#include "libtransmission/file.h"
+#include "libtransmission/log.h"
+#include "libtransmission/net.h"
+#include "libtransmission/peer-mgr.h" // for tr_peerMgrCompactToPex()
+#include "libtransmission/quark.h"
+#include "libtransmission/timer.h"
+#include "libtransmission/tr-assert.h"
+#include "libtransmission/tr-dht.h"
+#include "libtransmission/tr-strbuf.h"
+#include "libtransmission/variant.h"
+#include "libtransmission/utils.h" // for tr_time(), _()
 
 using namespace std::literals;
 
 // the dht library needs us to implement these:
 extern "C"
 {
-
     // This function should return true when a node is blacklisted.
     // We don't support using a blacklist with the DHT in Transmission,
     // since massive (ab)use of this feature could harm the DHT. However,
@@ -90,6 +90,7 @@ extern "C"
 
     int dht_sendto(int sockfd, void const* buf, int len, int flags, struct sockaddr const* to, int tolen)
     {
+        // NOLINTNEXTLINE(readability-redundant-casting)
         return static_cast<int>(sendto(sockfd, static_cast<char const*>(buf), len, flags, to, tolen));
     }
 
@@ -112,11 +113,11 @@ extern "C"
 class tr_dht_impl final : public tr_dht
 {
 private:
-    using Node = std::pair<tr_address, tr_port>;
+    using Node = tr_socket_address;
     using Nodes = std::deque<Node>;
     using Id = std::array<unsigned char, 20>;
 
-    enum class SwarmStatus
+    enum class SwarmStatus : uint8_t
     {
         Stopped,
         Broken,
@@ -131,28 +132,26 @@ public:
         , udp4_socket_{ udp4_socket }
         , udp6_socket_{ udp6_socket }
         , mediator_{ mediator }
-        , state_filename_{ tr_pathbuf{ mediator_.configDir(), "/dht.dat" } }
-        , announce_timer_{ mediator_.timerMaker().create([this]() { onAnnounceTimer(); }) }
-        , bootstrap_timer_{ mediator_.timerMaker().create([this]() { onBootstrapTimer(); }) }
-        , periodic_timer_{ mediator_.timerMaker().create([this]() { onPeriodicTimer(); }) }
+        , state_filename_{ tr_pathbuf{ mediator_.config_dir(), "/dht.dat" } }
+        , announce_timer_{ mediator_.timer_maker().create([this]() { on_announce_timer(); }) }
+        , bootstrap_timer_{ mediator_.timer_maker().create([this]() { on_bootstrap_timer(); }) }
+        , periodic_timer_{ mediator_.timer_maker().create([this]() { on_periodic_timer(); }) }
     {
         tr_logAddDebug(fmt::format("Starting DHT on port {port}", fmt::arg("port", peer_port.host())));
 
-        // load up the bootstrap nodes
-        if (tr_sys_path_exists(state_filename_.c_str()))
-        {
-            std::tie(id_, bootstrap_queue_) = loadState(state_filename_);
-        }
-        getNodesFromBootstrapFile(tr_pathbuf{ mediator_.configDir(), "/dht.bootstrap"sv }, bootstrap_queue_);
-        getNodesFromName("dht.transmissionbt.com", tr_port::fromHost(6881), bootstrap_queue_);
-        bootstrap_timer_->startSingleShot(100ms);
+        // init state from scratch, or load from state file if it exists
+        init_state(state_filename_);
+
+        get_nodes_from_bootstrap_file(tr_pathbuf{ mediator_.config_dir(), "/dht.bootstrap"sv }, bootstrap_queue_);
+        get_nodes_from_name("dht.transmissionbt.com", tr_port::from_host(6881), bootstrap_queue_);
+        bootstrap_timer_->start_single_shot(100ms);
 
         mediator_.api().init(udp4_socket_, udp6_socket_, std::data(id_), nullptr);
 
-        onAnnounceTimer();
-        announce_timer_->startRepeating(1s);
+        on_announce_timer();
+        announce_timer_->start_repeating(1s);
 
-        onPeriodicTimer();
+        on_periodic_timer();
     }
 
     tr_dht_impl(tr_dht_impl&&) = delete;
@@ -166,16 +165,16 @@ public:
 
         // Since we only save known good nodes,
         // only overwrite older data if we know enough nodes.
-        if (isReady(AF_INET) || isReady(AF_INET6))
+        if (is_ready(AF_INET) || is_ready(AF_INET6))
         {
-            saveState();
+            save_state();
         }
 
         mediator_.api().uninit();
         tr_logAddTrace("Done uninitializing DHT");
     }
 
-    void addNode(tr_address const& addr, tr_port port) override
+    void maybe_add_node(tr_address const& addr, tr_port port) override
     {
         if (addr.is_ipv4())
         {
@@ -195,18 +194,18 @@ public:
         }
     }
 
-    void handleMessage(unsigned char const* msg, size_t msglen, struct sockaddr* from, socklen_t fromlen) override
+    void handle_message(unsigned char const* msg, size_t msglen, struct sockaddr* from, socklen_t fromlen) override
     {
         auto const call_again_in_n_secs = periodic(msg, msglen, from, fromlen);
 
         // Being slightly late is fine,
         // and has the added benefit of adding some jitter.
         auto const interval = call_again_in_n_secs + std::chrono::milliseconds{ tr_rand_int(1000U) };
-        periodic_timer_->startSingleShot(interval);
+        periodic_timer_->start_single_shot(interval);
     }
 
 private:
-    [[nodiscard]] constexpr tr_socket_t udpSocket(int af) const noexcept
+    [[nodiscard]] constexpr tr_socket_t udp_socket(int af) const noexcept
     {
         switch (af)
         {
@@ -221,9 +220,9 @@ private:
         }
     }
 
-    [[nodiscard]] SwarmStatus swarmStatus(int family, int* const setme_node_count = nullptr) const
+    [[nodiscard]] SwarmStatus swarm_status(int family, int* const setme_node_count = nullptr) const
     {
-        if (udpSocket(family) == TR_BAD_SOCKET)
+        if (udp_socket(family) == TR_BAD_SOCKET)
         {
             if (setme_node_count != nullptr)
             {
@@ -261,25 +260,25 @@ private:
         return SwarmStatus::Good;
     }
 
-    [[nodiscard]] static constexpr auto isReady(SwarmStatus const status)
+    [[nodiscard]] static constexpr auto is_ready(SwarmStatus const status)
     {
         return status >= SwarmStatus::Firewalled;
     }
 
-    [[nodiscard]] bool isReady(int af) const noexcept
+    [[nodiscard]] bool is_ready(int af) const noexcept
     {
-        return isReady(swarmStatus(af));
+        return is_ready(swarm_status(af));
     }
 
-    [[nodiscard]] bool isReady() const noexcept
+    [[nodiscard]] bool is_ready() const noexcept
     {
-        return isReady(AF_INET) && isReady(AF_INET6);
+        return is_ready(AF_INET) && is_ready(AF_INET6);
     }
 
     ///
 
     // how long to wait between adding nodes during bootstrap
-    [[nodiscard]] static constexpr auto bootstrapInterval(size_t n_added)
+    [[nodiscard]] static constexpr auto bootstrap_interval(size_t n_added)
     {
         // Our DHT code is able to take up to 9 nodes in a row without
         // dropping any. After that, it takes some time to split buckets.
@@ -297,26 +296,26 @@ private:
         return 40s;
     }
 
-    void onBootstrapTimer()
+    void on_bootstrap_timer()
     {
         // Since we don't want to abuse our bootstrap nodes,
         // we don't ping them if the DHT is in a good state.
-        if (isReady() || std::empty(bootstrap_queue_))
+        if (is_ready() || std::empty(bootstrap_queue_))
         {
             return;
         }
 
         auto [address, port] = bootstrap_queue_.front();
         bootstrap_queue_.pop_front();
-        addNode(address, port);
+        maybe_add_node(address, port);
         ++n_bootstrapped_;
 
-        bootstrap_timer_->startSingleShot(bootstrapInterval(n_bootstrapped_));
+        bootstrap_timer_->start_single_shot(bootstrap_interval(n_bootstrapped_));
     }
 
     ///
 
-    [[nodiscard]] auto announceTorrent(tr_sha1_digest_t const& info_hash, int af, tr_port port)
+    [[nodiscard]] auto announce_torrent(tr_sha1_digest_t const& info_hash, int af, tr_port port)
     {
         auto const* dht_hash = reinterpret_cast<unsigned char const*>(std::data(info_hash));
         auto const rc = mediator_.api().search(dht_hash, port.host(), af, callback, this);
@@ -325,28 +324,28 @@ private:
         return announce_again_in_n_secs;
     }
 
-    void onAnnounceTimer()
+    void on_announce_timer()
     {
         // don't announce if the swarm isn't ready
-        if (swarmStatus(AF_INET) < SwarmStatus::Poor && swarmStatus(AF_INET6) < SwarmStatus::Poor)
+        if (swarm_status(AF_INET) < SwarmStatus::Poor && swarm_status(AF_INET6) < SwarmStatus::Poor)
         {
             return;
         }
 
         auto const now = tr_time();
-        for (auto const id : mediator_.torrentsAllowingDHT())
+        for (auto const id : mediator_.torrents_allowing_dht())
         {
             auto& times = announce_times_[id];
 
             if (auto& announce_after = times.ipv4_announce_after; announce_after < now)
             {
-                auto const announce_again_in_n_secs = announceTorrent(mediator_.torrentInfoHash(id), AF_INET, peer_port_);
+                auto const announce_again_in_n_secs = announce_torrent(mediator_.torrent_info_hash(id), AF_INET, peer_port_);
                 announce_after = now + std::chrono::seconds{ announce_again_in_n_secs }.count();
             }
 
             if (auto& announce_after = times.ipv6_announce_after; announce_after < now)
             {
-                auto const announce_again_in_n_secs = announceTorrent(mediator_.torrentInfoHash(id), AF_INET6, peer_port_);
+                auto const announce_again_in_n_secs = announce_torrent(mediator_.torrent_info_hash(id), AF_INET6, peer_port_);
                 announce_after = now + std::chrono::seconds{ announce_again_in_n_secs }.count();
             }
         }
@@ -354,14 +353,14 @@ private:
 
     ///
 
-    void onPeriodicTimer()
+    void on_periodic_timer()
     {
         auto const call_again_in_n_secs = periodic(nullptr, 0, nullptr, 0);
 
         // Being slightly late is fine,
         // and has the added benefit of adding some jitter.
         auto const interval = call_again_in_n_secs + std::chrono::milliseconds{ tr_rand_int(1000U) };
-        periodic_timer_->startSingleShot(interval);
+        periodic_timer_->start_single_shot(interval);
     }
 
     [[nodiscard]] std::chrono::seconds periodic(
@@ -383,11 +382,11 @@ private:
         {
             // paper over a bug in some DHT implementation that gives port 1.
             // Xref: https://github.com/transmission/transmission/issues/527
-            return candidate.port == tr_port::fromHost(1);
+            return candidate.socket_address.port_ == tr_port::from_host(1);
         };
 
         pex.erase(std::remove_if(std::begin(pex), std::end(pex), IsBadPex), std::end(pex));
-        return pex;
+        return std::move(pex);
     }
 
     static void callback(void* vself, int event, unsigned char const* info_hash, void const* data, size_t data_len)
@@ -399,25 +398,25 @@ private:
         if (event == DHT_EVENT_VALUES)
         {
             auto const pex = remove_bad_pex(tr_pex::from_compact_ipv4(data, data_len, nullptr, 0));
-            self->mediator_.addPex(hash, std::data(pex), std::size(pex));
+            self->mediator_.add_pex(hash, std::data(pex), std::size(pex));
         }
         else if (event == DHT_EVENT_VALUES6)
         {
             auto const pex = remove_bad_pex(tr_pex::from_compact_ipv6(data, data_len, nullptr, 0));
-            self->mediator_.addPex(hash, std::data(pex), std::size(pex));
+            self->mediator_.add_pex(hash, std::data(pex), std::size(pex));
         }
     }
 
     ///
 
-    void saveState() const
+    void save_state() const
     {
-        auto constexpr MaxNodes = int{ 300 };
-        auto constexpr PortLen = size_t{ 2 };
-        auto constexpr CompactAddrLen = size_t{ 4 };
-        auto constexpr CompactLen = size_t{ CompactAddrLen + PortLen };
-        auto constexpr Compact6AddrLen = size_t{ 16 };
-        auto constexpr Compact6Len = size_t{ Compact6AddrLen + PortLen };
+        static auto constexpr MaxNodes = 300;
+        static auto constexpr PortLen = tr_port::CompactPortBytes;
+        static auto constexpr CompactAddrLen = tr_address::CompactAddrBytes[TR_AF_INET];
+        static auto constexpr CompactLen = tr_socket_address::CompactSockAddrBytes[TR_AF_INET];
+        static auto constexpr Compact6AddrLen = tr_address::CompactAddrBytes[TR_AF_INET6];
+        static auto constexpr Compact6Len = tr_socket_address::CompactSockAddrBytes[TR_AF_INET6];
 
         auto sins4 = std::array<struct sockaddr_in, MaxNodes>{};
         auto sins6 = std::array<struct sockaddr_in6, MaxNodes>{};
@@ -427,8 +426,9 @@ private:
         tr_logAddTrace(fmt::format("Saving {} ({} + {}) nodes", n, num4, num6));
 
         tr_variant benc;
-        tr_variantInitDict(&benc, 3);
+        tr_variantInitDict(&benc, 4);
         tr_variantDictAddRaw(&benc, TR_KEY_id, std::data(id_), std::size(id_));
+        tr_variantDictAddInt(&benc, TR_KEY_id_timestamp, id_timestamp_);
 
         if (num4 > 0)
         {
@@ -460,65 +460,77 @@ private:
             tr_variantDictAddRaw(&benc, TR_KEY_nodes6, std::data(compact6), out6 - std::data(compact6));
         }
 
-        tr_variantToFile(&benc, TR_VARIANT_FMT_BENC, state_filename_);
-        tr_variantClear(&benc);
+        tr_variant_serde::benc().to_file(benc, state_filename_);
     }
 
-    [[nodiscard]] static std::pair<Id, Nodes> loadState(std::string_view filename)
+    void init_state(std::string_view filename)
     {
         // Note that DHT ids need to be distributed uniformly,
         // so it should be something truly random
-        auto id = tr_rand_obj<Id>();
+        id_ = tr_rand_obj<Id>();
+        id_timestamp_ = tr_time();
 
-        auto nodes = Nodes{};
-
-        if (auto dict = tr_variant{}; tr_variantFromFile(&dict, TR_VARIANT_PARSE_BENC, filename))
+        if (!tr_sys_path_exists(std::data(filename)))
         {
-            if (auto sv = std::string_view{};
-                tr_variantDictFindStrView(&dict, TR_KEY_id, &sv) && std::size(sv) == std::size(id))
-            {
-                std::copy(std::begin(sv), std::end(sv), std::begin(id));
-            }
-
-            size_t raw_len = 0U;
-            std::byte const* raw = nullptr;
-            if (tr_variantDictFindRaw(&dict, TR_KEY_nodes, &raw, &raw_len) && raw_len % 6 == 0)
-            {
-                auto* walk = raw;
-                auto const* const end = raw + raw_len;
-                while (walk < end)
-                {
-                    auto addr = tr_address{};
-                    auto port = tr_port{};
-                    std::tie(addr, walk) = tr_address::from_compact_ipv4(walk);
-                    std::tie(port, walk) = tr_port::fromCompact(walk);
-                    nodes.emplace_back(addr, port);
-                }
-            }
-
-            if (tr_variantDictFindRaw(&dict, TR_KEY_nodes6, &raw, &raw_len) && raw_len % 18 == 0)
-            {
-                auto* walk = raw;
-                auto const* const end = raw + raw_len;
-                while (walk < end)
-                {
-                    auto addr = tr_address{};
-                    auto port = tr_port{};
-                    std::tie(addr, walk) = tr_address::from_compact_ipv6(walk);
-                    std::tie(port, walk) = tr_port::fromCompact(walk);
-                    nodes.emplace_back(addr, port);
-                }
-            }
-
-            tr_variantClear(&dict);
+            return;
         }
 
-        return std::make_pair(id, nodes);
+        auto otop = tr_variant_serde::benc().parse_file(filename);
+        if (!otop)
+        {
+            return;
+        }
+
+        static auto constexpr CompactLen = tr_socket_address::CompactSockAddrBytes[TR_AF_INET];
+        static auto constexpr Compact6Len = tr_socket_address::CompactSockAddrBytes[TR_AF_INET6];
+        static auto constexpr IdTtl = time_t{ 30 * 24 * 60 * 60 }; // 30 days
+
+        auto& top = *otop;
+
+        if (auto t = int64_t{}; tr_variantDictFindInt(&top, TR_KEY_id_timestamp, &t) && t + IdTtl > id_timestamp_)
+        {
+            if (auto sv = std::string_view{};
+                tr_variantDictFindStrView(&top, TR_KEY_id, &sv) && std::size(sv) == std::size(id_))
+            {
+                id_timestamp_ = t;
+                std::copy(std::begin(sv), std::end(sv), std::begin(id_));
+            }
+        }
+
+        size_t raw_len = 0U;
+        std::byte const* raw = nullptr;
+        if (tr_variantDictFindRaw(&top, TR_KEY_nodes, &raw, &raw_len) && raw_len % CompactLen == 0)
+        {
+            auto* walk = raw;
+            auto const* const end = raw + raw_len;
+            while (walk < end)
+            {
+                auto addr = tr_address{};
+                auto port = tr_port{};
+                std::tie(addr, walk) = tr_address::from_compact_ipv4(walk);
+                std::tie(port, walk) = tr_port::from_compact(walk);
+                bootstrap_queue_.emplace_back(addr, port);
+            }
+        }
+
+        if (tr_variantDictFindRaw(&top, TR_KEY_nodes6, &raw, &raw_len) && raw_len % Compact6Len == 0)
+        {
+            auto* walk = raw;
+            auto const* const end = raw + raw_len;
+            while (walk < end)
+            {
+                auto addr = tr_address{};
+                auto port = tr_port{};
+                std::tie(addr, walk) = tr_address::from_compact_ipv6(walk);
+                std::tie(port, walk) = tr_port::from_compact(walk);
+                bootstrap_queue_.emplace_back(addr, port);
+            }
+        }
     }
 
     ///
 
-    static void getNodesFromBootstrapFile(std::string_view filename, Nodes& nodes)
+    static void get_nodes_from_bootstrap_file(std::string_view filename, Nodes& nodes)
     {
         auto in = std::ifstream{ std::string{ filename } };
         if (!in.is_open())
@@ -537,19 +549,20 @@ private:
 
             if (line_stream.bad() || std::empty(addrstr))
             {
-                tr_logAddWarn(fmt::format(
-                    _("Couldn't parse '{filename}' line: '{line}'"),
-                    fmt::arg("filename", filename),
-                    fmt::arg("line", line)));
+                tr_logAddWarn(
+                    fmt::format(
+                        fmt::runtime(_("Couldn't parse '{filename}' line: '{line}'")),
+                        fmt::arg("filename", filename),
+                        fmt::arg("line", line)));
             }
             else
             {
-                getNodesFromName(addrstr.c_str(), tr_port::fromHost(hport), nodes);
+                get_nodes_from_name(addrstr.c_str(), tr_port::from_host(hport), nodes);
             }
         }
     }
 
-    static void getNodesFromName(char const* name, tr_port port_in, Nodes& nodes)
+    static void get_nodes_from_name(char const* name, tr_port port_in, Nodes& nodes)
     {
         auto hints = addrinfo{};
         hints.ai_socktype = SOCK_DGRAM;
@@ -557,26 +570,25 @@ private:
         hints.ai_protocol = 0;
         hints.ai_flags = 0;
 
-        auto port_str = std::array<char, 16>{};
-        *fmt::format_to(std::data(port_str), FMT_STRING("{:d}"), port_in.host()) = '\0';
-
+        auto const port_str = fmt::format("{:d}", port_in.host());
         addrinfo* info = nullptr;
-        if (int const rc = getaddrinfo(name, std::data(port_str), &hints, &info); rc != 0)
+        if (int const rc = getaddrinfo(name, port_str.c_str(), &hints, &info); rc != 0)
         {
-            tr_logAddWarn(fmt::format(
-                _("Couldn't look up '{address}:{port}': {error} ({error_code})"),
-                fmt::arg("address", name),
-                fmt::arg("port", port_in.host()),
-                fmt::arg("error", gai_strerror(rc)),
-                fmt::arg("error_code", rc)));
+            tr_logAddWarn(
+                fmt::format(
+                    fmt::runtime(_("Couldn't look up '{address}:{port}': {error} ({error_code})")),
+                    fmt::arg("address", name),
+                    fmt::arg("port", port_in.host()),
+                    fmt::arg("error", gai_strerror(rc)),
+                    fmt::arg("error_code", rc)));
             return;
         }
 
         for (auto* infop = info; infop != nullptr; infop = infop->ai_next)
         {
-            if (auto addrport = tr_address::from_sockaddr(infop->ai_addr); addrport)
+            if (auto addrport = tr_socket_address::from_sockaddr(infop->ai_addr); addrport)
             {
-                nodes.emplace_back(addrport->first, addrport->second);
+                nodes.emplace_back(*addrport);
             }
         }
 
@@ -596,6 +608,7 @@ private:
     std::unique_ptr<libtransmission::Timer> const periodic_timer_;
 
     Id id_ = {};
+    int64_t id_timestamp_ = {};
 
     Nodes bootstrap_queue_;
     size_t n_bootstrapped_ = 0;
