@@ -1,4 +1,4 @@
-// This file Copyright © 2022-2023 Mnemosyne LLC.
+// This file Copyright © Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
@@ -11,7 +11,6 @@
 
 #include <memory>
 
-struct evbuffer;
 struct event;
 struct event_base;
 struct evhttp;
@@ -19,13 +18,6 @@ struct evhttp;
 namespace libtransmission::evhelpers
 {
 
-struct BufferDeleter
-{
-    void operator()(struct evbuffer* buf) const noexcept;
-};
-
-using evbuffer_unique_ptr = std::unique_ptr<struct evbuffer, BufferDeleter>;
-
 struct EventBaseDeleter
 {
     void operator()(struct event_base* evbase) const noexcept;
