@@ -1,4 +1,4 @@
-// This file Copyright © 2015-2023 Mnemosyne LLC.
+// This file Copyright © Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
@@ -17,22 +17,28 @@
 #include <event2/event.h>
 #include <event2/util.h>
 
-#include <fmt/core.h>
+#include <fmt/format.h>
 
 #define LIBTRANSMISSION_WATCHDIR_MODULE
 
-#include "transmission.h"
-#include "log.h"
-#include "net.h"
-#include "tr-assert.h"
-#include "utils.h"
-#include "watchdir-base.h"
+#include "libtransmission/transmission.h"
+
+#include "libtransmission/log.h"
+#include "libtransmission/net.h"
+#include "libtransmission/tr-assert.h"
+#include "libtransmission/utils.h"
+#include "libtransmission/watchdir-base.h"
 
 namespace libtransmission
 {
 namespace
 {
 
+constexpr bool to_bool(BOOL value) noexcept
+{
+    return value != FALSE;
+}
+
 BOOL tr_get_overlapped_result_ex(
     HANDLE handle,
     LPOVERLAPPED overlapped,
@@ -47,7 +53,7 @@ BOOL tr_get_overlapped_result_ex(
 
     if (!is_real_impl_valid)
     {
-        real_impl = (impl_t)GetProcAddress(GetModuleHandleW(L"kernel32.dll"), "GetOverlappedResultEx");
+        real_impl = reinterpret_cast<impl_t>(GetProcAddress(GetModuleHandleW(L"kernel32.dll"), "GetOverlappedResultEx"));
         is_real_impl_valid = true;
     }
 
@@ -113,12 +119,12 @@ public:
 
         if (notify_pipe_[0] != TR_BAD_SOCKET)
         {
-            evutil_closesocket(notify_pipe_[0]);
+            tr_net_close_socket(notify_pipe_[0]);
         }
 
         if (notify_pipe_[1] != TR_BAD_SOCKET)
         {
-            evutil_closesocket(notify_pipe_[1]);
+            tr_net_close_socket(notify_pipe_[1]);
         }
 
         if (fd_ != INVALID_HANDLE_VALUE)
@@ -133,8 +139,6 @@ private:
 
     void init(struct event_base* event_base)
     {
-        tr_net_init();
-
         auto const path = dirname();
         auto const wide_path = tr_win32_utf8_to_native(path);
         if (std::empty(wide_path))
@@ -143,14 +147,15 @@ private:
             return;
         }
 
-        if ((fd_ = CreateFileW(
-                 wide_path.c_str(),
-                 FILE_LIST_DIRECTORY,
-                 FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
-                 nullptr,
-                 OPEN_EXISTING,
-                 FILE_FLAG_BACKUP_SEMANTICS | FILE_FLAG_OVERLAPPED,
-                 nullptr)) == INVALID_HANDLE_VALUE)
+        fd_ = CreateFileW(
+            wide_path.c_str(),
+            FILE_LIST_DIRECTORY,
+            FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
+            nullptr,
+            OPEN_EXISTING,
+            FILE_FLAG_BACKUP_SEMANTICS | FILE_FLAG_OVERLAPPED,
+            nullptr);
+        if (fd_ == INVALID_HANDLE_VALUE)
         {
             tr_logAddError(fmt::format(_("Couldn't read '{path}'"), fmt::arg("path", path)));
             return;
@@ -158,7 +163,15 @@ private:
 
         overlapped_.Pointer = this;
 
-        if (!ReadDirectoryChangesW(fd_, buffer_, sizeof(buffer_), false, Win32WatchMask, nullptr, &overlapped_, nullptr))
+        if (!to_bool(ReadDirectoryChangesW(
+                fd_,
+                std::data(buffer_),
+                std::size(buffer_),
+                FALSE,
+                Win32WatchMask,
+                nullptr,
+                &overlapped_,
+                nullptr)))
         {
             tr_logAddError(fmt::format(_("Couldn't read '{path}'"), fmt::arg("path", path)));
             return;
@@ -167,10 +180,11 @@ private:
         if (evutil_socketpair(AF_INET, SOCK_STREAM, 0, std::data(notify_pipe_)) == -1)
         {
             auto const error_code = errno;
-            tr_logAddError(fmt::format(
-                _("Couldn't create pipe: {error} ({error_code})"),
-                fmt::arg("error", tr_strerror(error_code)),
-                fmt::arg("error_code", error_code)));
+            tr_logAddError(
+                fmt::format(
+                    _("Couldn't create pipe: {error} ({error_code})"),
+                    fmt::arg("error", tr_strerror(error_code)),
+                    fmt::arg("error_code", error_code)));
             return;
         }
 
@@ -178,10 +192,11 @@ private:
         if (event_ == nullptr)
         {
             auto const error_code = errno;
-            tr_logAddError(fmt::format(
-                _("Couldn't create event: {error} ({error_code})"),
-                fmt::arg("error", tr_strerror(error_code)),
-                fmt::arg("error_code", error_code)));
+            tr_logAddError(
+                fmt::format(
+                    _("Couldn't create event: {error} ({error_code})"),
+                    fmt::arg("error", tr_strerror(error_code)),
+                    fmt::arg("error_code", error_code)));
             return;
         }
 
@@ -189,7 +204,8 @@ private:
         bufferevent_setcb(event_, &Win32Watchdir::onBufferEvent, nullptr, nullptr, this);
         bufferevent_enable(event_, EV_READ);
 
-        thread_ = (HANDLE)_beginthreadex(nullptr, 0, Win32Watchdir::staticThreadFunc, this, 0, nullptr);
+        // NOLINTNEXTLINE(performance-no-int-to-ptr)
+        thread_ = reinterpret_cast<HANDLE>(_beginthreadex(nullptr, 0, Win32Watchdir::staticThreadFunc, this, 0, nullptr));
         if (thread_ == nullptr)
         {
             tr_logAddError(_("Couldn't create thread"));
@@ -204,22 +220,31 @@ private:
 
     unsigned int threadFunc()
     {
-        DWORD bytes_transferred;
+        DWORD bytes_transferred = 0;
 
-        while (tr_get_overlapped_result_ex(fd_, &overlapped_, &bytes_transferred, INFINITE, FALSE))
+        while (to_bool(tr_get_overlapped_result_ex(fd_, &overlapped_, &bytes_transferred, INFINITE, FALSE)))
         {
-            PFILE_NOTIFY_INFORMATION info = (PFILE_NOTIFY_INFORMATION)buffer_;
+            auto* info = reinterpret_cast<PFILE_NOTIFY_INFORMATION>(std::data(buffer_));
 
             while (info->NextEntryOffset != 0)
             {
-                *((BYTE**)&info) += info->NextEntryOffset;
+                *reinterpret_cast<BYTE**>(&info) += info->NextEntryOffset;
             }
 
-            info->NextEntryOffset = bytes_transferred - ((BYTE*)info - (BYTE*)buffer_);
+            info->NextEntryOffset = bytes_transferred -
+                (reinterpret_cast<BYTE*>(info) - reinterpret_cast<BYTE*>(std::data(buffer_)));
 
-            send(notify_pipe_[1], (char const*)buffer_, bytes_transferred, 0);
+            send(notify_pipe_[1], reinterpret_cast<char const*>(std::data(buffer_)), bytes_transferred, 0);
 
-            if (!ReadDirectoryChangesW(fd_, buffer_, sizeof(buffer_), FALSE, Win32WatchMask, nullptr, &overlapped_, nullptr))
+            if (!to_bool(ReadDirectoryChangesW(
+                    fd_,
+                    std::data(buffer_),
+                    std::size(buffer_),
+                    FALSE,
+                    Win32WatchMask,
+                    nullptr,
+                    &overlapped_,
+                    nullptr)))
             {
                 tr_logAddError(_("Couldn't read directory changes"));
                 return 0;
@@ -250,7 +275,7 @@ private:
 
         auto buffer = std::vector<char>{};
         buffer.resize(sizeof(FILE_NOTIFY_INFORMATION) + name_size);
-        PFILE_NOTIFY_INFORMATION ev = (PFILE_NOTIFY_INFORMATION)std::data(buffer);
+        auto* ev = reinterpret_cast<PFILE_NOTIFY_INFORMATION>(std::data(buffer));
 
         size_t const header_size = offsetof(FILE_NOTIFY_INFORMATION, FileName);
 
@@ -264,22 +289,24 @@ private:
                 break;
             }
 
-            if (nread == (size_t)-1)
+            if (nread == static_cast<size_t>(-1))
             {
                 auto const error_code = errno;
-                tr_logAddError(fmt::format(
-                    _("Couldn't read event: {error} ({error_code})"),
-                    fmt::arg("error", tr_strerror(error_code)),
-                    fmt::arg("error_code", error_code)));
+                tr_logAddError(
+                    fmt::format(
+                        _("Couldn't read event: {error} ({error_code})"),
+                        fmt::arg("error", tr_strerror(error_code)),
+                        fmt::arg("error_code", error_code)));
                 break;
             }
 
             if (nread != header_size)
             {
-                tr_logAddError(fmt::format(
-                    _("Couldn't read event: expected {expected_size}, got {actual_size}"),
-                    fmt::arg("expected_size", header_size),
-                    fmt::arg("actual_size", nread)));
+                tr_logAddError(
+                    fmt::format(
+                        _("Couldn't read event: expected {expected_size}, got {actual_size}"),
+                        fmt::arg("expected_size", header_size),
+                        fmt::arg("actual_size", nread)));
                 break;
             }
 
@@ -293,27 +320,29 @@ private:
             {
                 name_size = nleft;
                 buffer.resize(sizeof(FILE_NOTIFY_INFORMATION) + name_size);
-                ev = (PFILE_NOTIFY_INFORMATION)std::data(buffer);
+                ev = reinterpret_cast<PFILE_NOTIFY_INFORMATION>(std::data(buffer));
             }
 
             // consume entire name into buffer
             nread = bufferevent_read(event, &buffer[header_size], nleft);
-            if (nread == (size_t)-1)
+            if (nread == static_cast<size_t>(-1))
             {
                 auto const error_code = errno;
-                tr_logAddError(fmt::format(
-                    _("Couldn't read filename: {error} ({error_code})"),
-                    fmt::arg("error", tr_strerror(error_code)),
-                    fmt::arg("error_code", error_code)));
+                tr_logAddError(
+                    fmt::format(
+                        _("Couldn't read filename: {error} ({error_code})"),
+                        fmt::arg("error", tr_strerror(error_code)),
+                        fmt::arg("error_code", error_code)));
                 break;
             }
 
             if (nread != nleft)
             {
-                tr_logAddError(fmt::format(
-                    _("Couldn't read filename: expected {expected_size}, got {actual_size}"),
-                    fmt::arg("expected_size", nleft),
-                    fmt::arg("actual_size", nread)));
+                tr_logAddError(
+                    fmt::format(
+                        _("Couldn't read filename: expected {expected_size}, got {actual_size}"),
+                        fmt::arg("expected_size", nleft),
+                        fmt::arg("actual_size", nread)));
                 break;
             }
 
@@ -331,7 +360,7 @@ private:
 
     HANDLE fd_ = INVALID_HANDLE_VALUE;
     OVERLAPPED overlapped_ = {};
-    DWORD buffer_[8 * 1024 / sizeof(DWORD)];
+    std::array<DWORD, 8 * 1024 / sizeof(DWORD)> buffer_ = {};
     std::array<evutil_socket_t, 2> notify_pipe_{ static_cast<evutil_socket_t>(-1), static_cast<evutil_socket_t>(-1) };
     struct bufferevent* event_ = nullptr;
     HANDLE thread_ = {};
@@ -343,9 +372,9 @@ std::unique_ptr<Watchdir> Watchdir::create(
     std::string_view dirname,
     Callback callback,
     TimerMaker& timer_maker,
-    struct event_base* event_base)
+    struct event_base* evbase)
 {
-    return std::make_unique<Win32Watchdir>(dirname, std::move(callback), timer_maker, event_base);
+    return std::make_unique<Win32Watchdir>(dirname, std::move(callback), timer_maker, evbase);
 }
 
 } // namespace libtransmission
