@@ -1,10 +1,12 @@
-// This file Copyright © 2012-2023 Mnemosyne LLC.
+// This file Copyright © Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
 
 #pragma once
 
+#include <libtransmission/transmission.h>
+
 #include <QVariant>
 
 class CustomVariantType
@@ -15,7 +17,10 @@ public:
         TrackerStatsList = QMetaType::User,
         PeerList,
         FileList,
-        FilterModeType,
-        SortModeType
+        ShowModeType,
+        SortModeType,
+        EncryptionModeType,
     };
 };
+
+Q_DECLARE_METATYPE(tr_encryption_mode)
