@@ -1,4 +1,4 @@
-// This file Copyright © 2009-2022 Mnemosyne LLC.
+// This file Copyright © Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
@@ -19,6 +19,7 @@ void SessionDialog::accept()
     prefs_.set(Prefs::SESSION_REMOTE_AUTH, ui_.authCheck->isChecked());
     prefs_.set(Prefs::SESSION_REMOTE_USERNAME, ui_.usernameEdit->text());
     prefs_.set(Prefs::SESSION_REMOTE_PASSWORD, ui_.passwordEdit->text());
+    prefs_.set(Prefs::SESSION_REMOTE_URL_BASE_PATH, ui_.rpcEdit->text());
     session_.restart();
     BaseDialog::accept();
 }
@@ -44,9 +45,9 @@ void SessionDialog::resensitize() const
 ***/
 
 SessionDialog::SessionDialog(Session& session, Prefs& prefs, QWidget* parent)
-    : BaseDialog(parent)
-    , session_(session)
-    , prefs_(prefs)
+    : BaseDialog{ parent }
+    , session_{ session }
+    , prefs_{ prefs }
 {
     ui_.setupUi(this);
 
@@ -72,5 +73,8 @@ SessionDialog::SessionDialog(Session& session, Prefs& prefs, QWidget* parent)
     ui_.passwordEdit->setText(prefs.get<QString>(Prefs::SESSION_REMOTE_PASSWORD));
     auth_widgets_ << ui_.passwordLabel << ui_.passwordEdit;
 
+    ui_.rpcEdit->setText(prefs.get<QString>(Prefs::SESSION_REMOTE_URL_BASE_PATH));
+    remote_widgets_ << ui_.rpcLabel << ui_.rpcEdit;
+
     resensitize();
 }
