@@ -1,4 +1,4 @@
-// This file Copyright © 2009-2022 Mnemosyne LLC.
+// This file Copyright © Mnemosyne LLC.
 // It may be used under GPLv2 (SPDX: GPL-2.0-only), GPLv3 (SPDX: GPL-3.0-only),
 // or any future license endorsed by Mnemosyne LLC.
 // License text can be found in the licenses/ folder.
@@ -7,8 +7,6 @@
 
 #include <QTimer>
 
-#include <libtransmission/tr-macros.h>
-
 #include "BaseDialog.h"
 #include "ui_StatsDialog.h"
 
@@ -17,10 +15,13 @@ class Session;
 class StatsDialog : public BaseDialog
 {
     Q_OBJECT
-    TR_DISABLE_COPY_MOVE(StatsDialog)
 
 public:
     explicit StatsDialog(Session&, QWidget* parent = nullptr);
+    StatsDialog(StatsDialog&&) = delete;
+    StatsDialog(StatsDialog const&) = delete;
+    StatsDialog& operator=(StatsDialog&&) = delete;
+    StatsDialog& operator=(StatsDialog const&) = delete;
 
     // QWidget
     void setVisible(bool visible) override;
